


#include <portb.h>
#include "sd.h"


SD_CSD gblCSDReg;

SD_RESPONSE sd_write(unsigned char, long);
SD_STATUS sd_sector_read(long, unsigned char*);
SD_STATUS sd_get_csd();

//extern struct Bf;

extern SD_CSD gblCSDReg;
extern SD_CID gblCIDReg;

void sd_init_host(void) {

	PORTBbits.RB0 = 0;
	LATBbits.LATB0 = 0;

	TRISBbits.TRISB0 = 1;	// Card Detect

//	INTCON2bits.NOT_RBPU = 0;
	OpenPORTB(PORTB_CHANGE_INT_OFF & PORTB_PULLUPS_ON);

	TRISAbits.TRISA5 = 0;	// Card Select
}


SD_STATUS sd_init(void) {

	SD_RESPONSE response;
	SD_STATUS status;
	unsigned int c;

	// On ouvre le SPI
	OpenSPI(SPI_FOSC_64, MODE_11, SMPMID);

	Delay10KTCYx(100);

	// Initialisation avec au moins 74 cycles d'horloge
	for (c = 0; c < 10; c++)
		WriteSPI(0xFF);

	// Chaque transaction, on passe CS  0
	SD_CS = 0;

	Delay10KTCYx(1);

	// On envoie 0x40 0x00 0x00 0x00 0x00 0x95 pour passer en mode IDLE
    response = sd_write(GO_IDLE_STATE, 0x00);

	if (response.r1.value == SD_BAD_RESPONSE) {
        status = SD_STATUS_INIT_COM_FAILURE;
        goto _sd_init_error;
    }

    if (response.r1.value != 0x01) {
        status = SD_STATUS_NOT_INIT_FAILURE;
        goto _sd_init_error;
    }   

    c = 0xFFF;	// Environ 1 seconde d'attente
    
	/*	On envoie la SEND_OP_COND (Card Initialization Process) jusqu' ce que la SD rponde OK
		Pass un certain TimeOut, on sort
	 */
	do {
		response = sd_write(SEND_OP_COND, 0x00);
		c--;
	} while (response.r1.value != 0x00 && c != 0);

    // see if it failed        
    if (c == 0) {
        status = SD_STATUS_INIT_TIMEOUT;      // we have not got anything back from the card 
        goto _sd_init_error;
    } else {
		// get the CSD register before increasing the speed page 4-8 of sd card manual last line
		status = sd_get_csd(&gblCSDReg);
		if (!status)
			OpenSPI(SPI_FOSC_4, MODE_11, SMPMID);       //Increase clock speed
		else 
			status = SD_STATUS_TYPE_INVALID;
	}

	response = sd_write(SET_BLOCKLEN, 512);
/*
	for(c = 0xFF; c > 0 && sd_sector_read(0x00, &Bf.BK_Buffer) != SD_STATUS_VALID; c--);

	if (c == 0) {      
		status = SD_STATUS_NOT_INIT_FAILURE;
		goto _sd_init_error; 
	}
*/
_sd_init_error:
	SD_CS = 1;
	return;
}

unsigned char sd_read(void)
{
    SSPBUF = 0xFF;                              //Data Out - Logic ones
    while(!SSPSTATbits.BF);                     //Wait until cycle complete
    return(SSPBUF);                             //Return with byte read 
}


/*	Envoie une commande
	@param cmd La commande  envoyer !
	@param 
 */
SD_RESPONSE sd_write(unsigned char cmd, long arg) {

	SD_CMD_PACKET command;
	SD_RESPONSE response;
	long timeout = 8;

	SD_CS = 0;

	command.cmd = sd_cmdtable[cmd].cmdCode;
	command.arg = arg;
	command.crc = sd_cmdtable[cmd].CRC;

	WriteSPI(command.cmd);
	WriteSPI(command.arg3);
	WriteSPI(command.arg2);
	WriteSPI(command.arg1);
	WriteSPI(command.arg0);
	WriteSPI(command.crc);

	switch (sd_cmdtable[cmd].responseType) {
		case R1:
		case R1b:

			do {
				response.r1.value = sd_read();
				timeout--;
			} while ((response.r1.value == 0xFF) && (timeout != 0));

			break;

		case R2:
			break;
	}

	WriteSPI(0xFF);

    if(!(sd_cmdtable[cmd].moreDataExpected))
        SD_CS = 1;

	return response;
}




/*	Lit un secteur
	@param sector_addr Numro du secteur
	@param buffer Buffer ou stocker le rsultat
*/
SD_STATUS sd_sector_read(long sector_addr, unsigned char* buffer)
{
	int index;
	SD_RESPONSE    response;
	unsigned char data_token;
	SD_STATUS status = SD_STATUS_VALID;
	
	response = sd_write(READ_SINGLE_BLOCK, (sector_addr << 9));
	
	// Make sure the command was accepted
	if(response.r1.value != 0x00)
		status = SD_STATUS_BAD_CMD;
	else {
		index = 0x2FF;                                     
	

		do {
			data_token = sd_read();
			index--;
		} while ((data_token == 0xFF) && (index != 0));
	

		if ((index == 0) || (data_token != SD_DATA_START_TOKEN))
			status = SD_STATUS_TIMEOUT;
		else
			{
				for (index = 0; index < 512; index++)
					*buffer++ = sd_read();
				
				WriteSPI(0xFF);
				WriteSPI(0xFF);
			}
			
			WriteSPI(0xFF);                       //Required clocking (see spec)
	}
	
	SD_CS = 1;
	
	return(status);
}


/*	Ecrit le contenu du buffer  l'emplacement spcifi
	@param sector_addr Numro du secteur
	@param buffer Buffer ou stocker le rsultat
 */
SD_STATUS sd_sector_write(long sector_addr, unsigned char* buffer) {

	unsigned char data_response;
	int index;
	SD_RESPONSE    response;
	unsigned char data_token;
	SD_STATUS status = SD_STATUS_VALID;
	
	response = sd_write(WRITE_SINGLE_BLOCK, (sector_addr << 9));
	
	// Make sure the command was accepted
	if(response.r1.value != 0x00)
		status = SD_STATUS_BAD_CMD;
	else {

		WriteSPI(SD_DATA_START_TOKEN);               //Send data start token
		
		for(index = 0; index < 512; index++)      //Send 512 bytes of data
			WriteSPI(buffer[index]);
		
		WriteSPI(0xFF);		// On envoie 2 octets de CRC
		WriteSPI(0xFF);
		
		data_response = sd_read();              //Read response
		
		if ((data_response & 0x0F) != DATA_ACCEPTED) {
			status = SD_STATUS_DATA_REJECTED;
		} else {

			index = 0;	// using i as a timeout counter
			
			do {		// Wait for write completion
				data_response = sd_read();
				index++;
			} while ((data_response == 0x00) && (index != 0));
			
			if (index == 0)                        //if timeout first
			status = SD_STATUS_TIMEOUT;
		}
			
		WriteSPI(0xFF);        
	}

	SD_CS = 1;
	
  	return (status);
}


/*	Renvoie si une SD  t insre
	@return renvoie true si la carte est prsente
 */
unsigned char sd_detect(void) {
	return (!SD_CD);
}


/*	Vient rcuprer le contenu du CSD (Card Specific Data)
	@param buffer Buffer ou stocker le rsultat
 */
SD_STATUS sd_get_csd(unsigned char * buffer)
{
	unsigned char data_token;
	int index, timeout = 0x2ff;
	SD_RESPONSE response;
	SD_STATUS status = SD_STATUS_VALID;
	SD_CMD_PACKET CmdPacket;
	
	SD_CS = 0;
	
	CmdPacket.cmd = sd_cmdtable[SEND_CSD].cmdCode;
	CmdPacket.arg = 0x00;
	CmdPacket.crc = sd_cmdtable[SEND_CSD].CRC;
	
	// On envoie tout a !
	WriteSPI(CmdPacket.cmd);
	WriteSPI(CmdPacket.arg3);
	WriteSPI(CmdPacket.arg2);
	WriteSPI(CmdPacket.arg1);
	WriteSPI(CmdPacket.arg0);
	WriteSPI(CmdPacket.crc);

	do {
		response.r1.value = sd_read();
		timeout--;
	} while ((response.r1.value == 0xFF) && (timeout != 0));
	
	// Make sure the command was accepted
	if(response.r1.value != 0x00) {
		status = SD_STATUS_BAD_CMD;
	} else {
		index = 0x2FF;                                     
		
		// Now, must wait for the start token of data block   
		do {
			data_token = sd_read();
			index--;
		} while ((data_token == 0xFF) && (index != 0));
		
		// Hopefully that zero is the datatoken
		if ((index == 0) || (data_token != SD_DATA_START_TOKEN))
			status = SD_STATUS_TIMEOUT;
		else {
			for(index = 0; index < 16; index++) {
				*buffer++ = sd_read();
			}
		}
	
		WriteSPI(0xFF);		// Required clocking (see spec)
	}
	
	SD_CS = 1;
	return status;
}


/*	Vient rcuprer le contenu du CSD (Card Specific Data)
	@param buffer Buffer ou stocker le rsultat
 */
/*
SD_STATUS sd_get_cid(unsigned char * buffer)
{
	unsigned char data_token;
	int index, timeout = 0x2ff;
	SD_RESPONSE response;
	SD_STATUS status = SD_STATUS_VALID;
	SD_CMD_PACKET CmdPacket;
	
	SD_CS = 0;
	
	CmdPacket.cmd = sd_cmdtable[SEND_CID].cmdCode;
	CmdPacket.arg = 0x00;
	CmdPacket.crc = sd_cmdtable[SEND_CID].CRC;

	// On envoie tout a !
	WriteSPI(CmdPacket.cmd);
	WriteSPI(CmdPacket.arg3);
	WriteSPI(CmdPacket.arg2);
	WriteSPI(CmdPacket.arg1);
	WriteSPI(CmdPacket.arg0);
	WriteSPI(CmdPacket.crc);

	do {
		response.r1.value = sd_read();
		timeout--;
	} while ((response.r1.value == 0xFF) && (timeout != 0));
	
	// Make sure the command was accepted
	if(response.r1.value != 0x00) {
		status = SD_STATUS_BAD_CMD;
	} else {
		index = 0x2FF;                                     
		
		// Now, must wait for the start token of data block   
		do
		{
			data_token = sd_read();
			index--;
		} while ((data_token == 0xFF) && (index != 0));
		
		// Hopefully that zero is the datatoken
		if ((index == 0) || (data_token != SD_DATA_START_TOKEN))
			status = SD_STATUS_TIMEOUT;
		else {
			for(index = 0; index < 16; index++) {
				*buffer++ = sd_read();
			}
		}
	
		WriteSPI(0xFF);		// Required clocking (see spec)
	}
	
	SD_CS = 1;
	return status;
}
*/
