



#define SD_CD		PORTBbits.RB0	// Card Detect
#define SD_CS		LATAbits.LATA5	// Card Select


#define SD_DATA_START_TOKEN	0xFE
#define DATA_ACCEPTED		0b00000101

/*	Les diffrents types de rponses
 */
typedef enum {
	R1,		// Standard response (48bits)
	R1b,	// Same as R1 with additional busy signal transmitted on the data line
	R2,		// The content of the CIDE register is sent as a response to CMD2 and CMD10 (136bits)
	R3		// Only for READ_OCR command
} SD_RESP;


/*	Structure des commandes
 */
typedef struct {
    unsigned char	cmdCode;			// Command number
    unsigned char	CRC;				// CRC
    SD_RESP			responseType;		// Responde type
    unsigned char	moreDataExpected;	// True if more data expected
} SD_sCMD;


/*	Status
 */
#define SD_BAD_RESPONSE    0xFF
#define SD_ILLEGAL_CMD     0x04
#define SD_GOOD_CMD        0x00

typedef enum {
    SD_STATUS_VALID = 0,				// Everything is golden
    SD_STATUS_INIT_COM_FAILURE,		// Communication has never been established with card
    SD_STATUS_NOT_INIT_FAILURE,		// Card did not go into an initialization phase
    SD_STATUS_INIT_TIMEOUT,			// Card initialization has timedout
    SD_STATUS_TYPE_INVALID,			// Card type was not able to be defined
    SD_STATUS_BAD_CMD,				// Card did not reconized the command
    SD_STATUS_TIMEOUT,				// Card timedout during a read, write or erase sequence   
    SD_STATUS_CRC_ERROR,			// A CRC error has occurred during a read, data should be invalidated
    SD_STATUS_DATA_REJECTED,		// Card and data sent's CRC did not match
    SD_STATUS_ERASE_TIMEDOUT		// Erase took longer than it should have 
} SD_STATUS;


/*	Les commandes
 */
typedef enum
{
	GO_IDLE_STATE,		// CMD0
	SEND_OP_COND,		// CMD1
	SEND_CSD,
	SEND_CID,
/*
	STOP_TRANSMISSION,
	SEND_STATUS,
*/
	SET_BLOCKLEN,
	READ_SINGLE_BLOCK,
/*
	READ_MULTI_BLOCK,
*/
	WRITE_SINGLE_BLOCK
/*
	WRITE_MULTI_BLOCK,
	TAG_SECTOR_START,
	TAG_SECTOR_END,
	UNTAG_SECTOR,
	TAG_ERASE_GRP_START,
	TAG_ERASE_GRP_END,
	UNTAG_ERASE_GRP,
	ERASE,
	LOCK_UNLOCK,
	SD_APP_OP_COND,
	APP_CMD,
	READ_OCR,
	CRC_ON_OFF
*/
} SD_CMD;


#define MOREDATA    !0
#define NODATA      0


#define SD_CMD_GO_IDLE_STATE		0x40
#define SD_CMD_SEND_OP_COND			0x41
#define	SD_CMD_SEND_CSD				0x49
#define	SD_CMD_SEND_CID				0x4A

#define	SD_CMD_SET_BLOCKLEN			0x50

#define SD_CMD_READ_SINGLE_BLOCK	0x51

#define	SD_WRITE_SINGLE_BLOCK		0x58

/*********************************************************************
 * sdmmc_cmdtable
 * - Provides information for all the sdmmc commands that we support
 * 
 * Notes: We turn off the CRC as soon as possible, so the commands with
 *        0xFF don't need to be calculated in runtime 
 *********************************************************************/
const rom SD_sCMD sd_cmdtable[] =
{
    // cmd                  crc   response
    {SD_CMD_GO_IDLE_STATE,		0x95, R1, NODATA},
    {SD_CMD_SEND_OP_COND,		0xF9, R1, NODATA},
	{SD_CMD_SEND_CSD,			0xAF, R1, MOREDATA},
    {SD_CMD_SEND_CID,			0x1B, R1, MOREDATA},
/*
    {cmdSTOP_TRANSMISSION,  0xC3, R1, NODATA},
    {cmdSEND_STATUS,        0xAF, R2, NODATA},
*/
    {SD_CMD_SET_BLOCKLEN,		0xFF, R1, NODATA},
    {SD_CMD_READ_SINGLE_BLOCK,	0xFF, R1, MOREDATA},
/*
    {cmdREAD_MULTI_BLOCK,   0xFF, R1, MOREDATA},
*/
    {SD_WRITE_SINGLE_BLOCK, 0xFF, R1, MOREDATA},
/*
    {cmdWRITE_MULTI_BLOCK,  0xFF, R1, MOREDATA}, 
    {cmdTAG_SECTOR_START,   0xFF, R1, NODATA},
    {cmdTAG_SECTOR_END,     0xFF, R1, NODATA},
    {cmdUNTAG_SECTOR,       0xFF, R1, NODATA},
    {cmdTAG_ERASE_GRP_START, 0xFF, R1, NODATA},
    {cmdTAG_ERASE_GRP_END,  0xFF, R1, NODATA},
    {cmdUNTAG_ERASE_GRP,    0xFF, R1, NODATA},
    {cmdERASE,              0xDF, R1b, NODATA},
    {cmdLOCK_UNLOCK,        0x89, R1b, NODATA},  
    {cmdSD_APP_OP_COND,     0xE5, R1, NODATA},
    {cmdAPP_CMD,            0x73, R1, NODATA},
    {cmdREAD_OCR,           0x25, R3, NODATA},
    {cmdCRC_ON_OFF,         0x25, R1, NODATA}
*/
};


typedef union
{
	unsigned char value;
	struct
	{
		unsigned IN_IDLE_STATE:1;
		unsigned ERASE_RESET:1;
		unsigned ILLEGAL_CMD:1;
		unsigned CRC_ERR:1;
		unsigned ERASE_SEQ_ERR:1;
		unsigned ADDRESS_ERR:1;
		unsigned PARAM_ERR:1;
		unsigned B7:1;
	};
} SD_RESPONSE_R1;


typedef union
{
	unsigned int value;
	struct
	{
		unsigned CARD_IS_LOCKED:1;
		unsigned WP_ERASE_SKIP_LK_FAIL:1;
		unsigned ERROR:1;
		unsigned CC_ERROR:1;
		unsigned CARD_ECC_FAIL:1;
		unsigned WP_VIOLATION:1;
		unsigned ERASE_PARAM:1;
		unsigned OUTRANGE_CSD_OVERWRITE:1;


	};
} SD_RESPONSE_R2;


typedef union
{
	SD_RESPONSE_R1  r1;  
	SD_RESPONSE_R2  r2;
} SD_RESPONSE;



typedef union
{
    struct
    {
        unsigned char crc;
        unsigned char arg0;
        unsigned char arg1;
        unsigned char arg2;
        unsigned char arg3;
        unsigned char cmd;
    };
    struct
    {
        unsigned CRC7:7;
        long arg;
        unsigned char command;
    };
} SD_CMD_PACKET;




typedef union
{
/*
    struct
    {
        DWORD _u320;
        DWORD _u321;
        DWORD _u322;
        DWORD _u323;
    };
*/
/*
    struct
    {
        unsigned char _byte[16];
    };
*/
    struct
    {
        unsigned CSD_STRUCTURE      :2;

        unsigned RESERVED_3         :6;
        
        unsigned TAAC               :8;
        
        unsigned NSAC               :8;
        
        unsigned TRAN_SPEED         :8;
        
        unsigned CCC_H              :4;
        unsigned char CCC_L          ;

        unsigned READ_BL_LEN        :4;
        
        unsigned READ_BL_PARTIAL    :1;

        unsigned WRITE_BLK_MISALIGN :1;
        unsigned READ_BLK_MISALIGN  :1;
        unsigned DSR_IMP            :1;
        unsigned RESERVED_2         :2;
//        unsigned C_SIZE_U           :2;
        
        unsigned C_SIZE_H           :4;      
        unsigned C_SIZE_L           :8;

        unsigned VDD_R_CURR_MIN     :3;
        unsigned VDD_R_CURR_MAX     :3;
        
        unsigned VDD_W_CUR_MIN      :3;
        unsigned VDD_W_CURR_MAX     :3;
        unsigned C_SIZE_MULT	    :3;
                
        unsigned ERASE_BLK_EN		:1;
        unsigned SECTOR_SIZE        :7;

        unsigned WP_GRP_SIZE        :7;

        unsigned WP_GRP_ENABLE      :1; //bit 016 - 031

        unsigned RESERVED_1         :2;

        unsigned R2W_FACTOR         :3;
        unsigned WRITE_BL_LEN       :4;
       
        unsigned WRITE_BL_PARTIAL   :1;
        unsigned RESERVED_4         :5;
        
        unsigned FILE_FORMAT_GRP    :1; //bit 008 - 015
        unsigned COPY               :1;
        unsigned PERM_WRITE_PROTECT :1;
        unsigned TMP_WRITE_PROTECT  :1;
        unsigned FILE_FORMAT        :2;
        unsigned ECC                :2;
        
        unsigned CRC                :7; //bit 000 - 007
        unsigned NOT_USED           :1;
    };
} SD_CSD;


typedef struct {
	unsigned char 		MID;		// Manufacture ID
	unsigned int		OID;		// OEM/Application ID
	unsigned char		PNM[5];		// Product Name
	unsigned char 		PRV;		// Product Revision
	unsigned long		PSN;		// Serial Number (PSN)
	unsigned			RSV:4;		// Reserved
	unsigned char 		MDT;		// Manufacturing Date Code (BCD)
	unsigned 			MDT2:4;
	unsigned 			CRC:7;
	unsigned 			NOT_USED:1;	// Always "1"
} SD_CID;


